@@ -1,7 +1,9 @@
 /*
  * DMNTK - Decision Model and Notation Toolkit
  *
- * Copyright 2018-2021 Dariusz Depta Engos Software <dariusz.depta@engos.software>
+ * MIT license
+ *
+ * Copyright (c) 2018-2021 Dariusz Depta Engos Software
  *
  * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
  * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
@@ -10,11 +12,27 @@
  * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
  * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
  * SOFTWARE.
+ *
+ * Apache license, Version 2.0
+ *
+ * Copyright (c) 2018-2021 Dariusz Depta Engos Software
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *     http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
  */
 
 //! `FEEL` years and months durations.
 
-use self::errors::*;
+use crate::temporal::errors::*;
 use dmntk_common::DmntkError;
 use regex::Regex;
 use std::convert::TryFrom;
@@ -28,7 +46,7 @@ lazy_static! {
 
 /// Years and months duration in `FEEL`.
 /// Holds the number of months in the duration.
-#[derive(Debug, Clone, PartialEq)]
+#[derive(Debug, Default, Clone, PartialEq, PartialOrd)]
 pub struct FeelYearsAndMonthsDuration(i64);
 
 /// Number of months in a year.
@@ -61,28 +79,6 @@ impl FeelYearsAndMonthsDuration {
   }
 }
 
-impl std::cmp::PartialOrd for FeelYearsAndMonthsDuration {
-  fn partial_cmp(&self, other: &Self) -> Option<std::cmp::Ordering> {
-    self.0.partial_cmp(&other.0)
-  }
-
-  fn lt(&self, other: &Self) -> bool {
-    self.0 < other.0
-  }
-
-  fn le(&self, other: &Self) -> bool {
-    self.0 <= other.0
-  }
-
-  fn gt(&self, other: &Self) -> bool {
-    self.0 > other.0
-  }
-
-  fn ge(&self, other: &Self) -> bool {
-    self.0 >= other.0
-  }
-}
-
 impl std::fmt::Display for FeelYearsAndMonthsDuration {
   fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
     let sign = if self.0 < 0 { "-" } else { "" };
@@ -124,38 +120,7 @@ impl TryFrom<&str> for FeelYearsAndMonthsDuration {
         return Ok(FeelYearsAndMonthsDuration(total_months));
       }
     }
-    Err(invalid_years_and_months_duration_literal(value.to_string()))
-  }
-}
-
-/// Definitions of years and months duration errors.
-pub mod errors {
-  use dmntk_common::DmntkError;
-
-  /// Years and months duration errors.
-  #[derive(Debug, PartialEq)]
-  enum YmDurationError {
-    InvalidYearsAndMonthsDurationLiteral(String),
-  }
-
-  impl From<YmDurationError> for DmntkError {
-    fn from(e: YmDurationError) -> Self {
-      DmntkError::new("YmDurationError", &format!("{}", e))
-    }
-  }
-
-  impl std::fmt::Display for YmDurationError {
-    fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
-      match self {
-        YmDurationError::InvalidYearsAndMonthsDurationLiteral(literal) => {
-          write!(f, "invalid years and months duration literal: {}", literal)
-        }
-      }
-    }
-  }
-
-  pub fn invalid_years_and_months_duration_literal(literal: String) -> DmntkError {
-    YmDurationError::InvalidYearsAndMonthsDurationLiteral(literal).into()
+    Err(err_invalid_years_and_months_duration_literal(value))
   }
 }
 
@@ -223,4 +188,104 @@ mod tests {
     equals_str("P2Y3M", 2, 3);
     equals_str("-P2Y3M", -2, -3);
   }
+
+  #[test]
+  fn test_eq_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    assert!((a == b));
+    let a = FeelYearsAndMonthsDuration::new_ym(2, 3);
+    let b = FeelYearsAndMonthsDuration::new_ym(2, 3);
+    assert!((a == b));
+    let a = FeelYearsAndMonthsDuration::new_ym(-3, 4);
+    let b = FeelYearsAndMonthsDuration::new_ym(-3, 4);
+    assert!((a == b));
+    let a = FeelYearsAndMonthsDuration::new_m(0);
+    let b = FeelYearsAndMonthsDuration::new_m(0);
+    assert!((a == b));
+    let a = FeelYearsAndMonthsDuration::new_m(-0);
+    let b = FeelYearsAndMonthsDuration::new_m(0);
+    assert!((a == b));
+    let a = FeelYearsAndMonthsDuration::new_m(-15);
+    let b = FeelYearsAndMonthsDuration::new_m(-15);
+    assert!((a == b));
+  }
+
+  #[test]
+  fn test_ne_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    assert!((a != b));
+    let a = FeelYearsAndMonthsDuration::new_m(1);
+    let b = FeelYearsAndMonthsDuration::new_m(0);
+    assert!((a != b));
+  }
+
+  #[test]
+  fn test_lt_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    assert!(a < b);
+    let a = FeelYearsAndMonthsDuration::new_m(4);
+    let b = FeelYearsAndMonthsDuration::new_m(5);
+    assert!(a < b);
+  }
+
+  #[test]
+  fn test_le_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    assert!(a <= b);
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    assert!(a <= b);
+    let a = FeelYearsAndMonthsDuration::new_m(12);
+    let b = FeelYearsAndMonthsDuration::new_m(18);
+    assert!(a <= b);
+    let a = FeelYearsAndMonthsDuration::new_m(16);
+    let b = FeelYearsAndMonthsDuration::new_m(16);
+    assert!(a <= b);
+  }
+
+  #[test]
+  fn test_gt_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    assert!(a > b);
+    let a = FeelYearsAndMonthsDuration::new_m(5);
+    let b = FeelYearsAndMonthsDuration::new_m(4);
+    assert!(a > b);
+  }
+
+  #[test]
+  fn test_ge_should_pass() {
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 0);
+    assert!(a >= b);
+    let a = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    let b = FeelYearsAndMonthsDuration::new_ym(0, 1);
+    assert!(a >= b);
+    let a = FeelYearsAndMonthsDuration::new_m(18);
+    let b = FeelYearsAndMonthsDuration::new_m(12);
+    assert!(a >= b);
+    let a = FeelYearsAndMonthsDuration::new_m(16);
+    let b = FeelYearsAndMonthsDuration::new_m(16);
+    assert!(a >= b);
+  }
+
+  #[test]
+  fn test_abs_should_pass() {
+    let duration = FeelYearsAndMonthsDuration::try_from("P2Y3M").unwrap();
+    assert_eq!("P2Y3M", duration.abs().to_string());
+    let duration = FeelYearsAndMonthsDuration::try_from("-P2Y3M").unwrap();
+    assert_eq!("P2Y3M", duration.abs().to_string());
+  }
+
+  #[test]
+  fn test_properties() {
+    let duration = FeelYearsAndMonthsDuration::try_from("P2Y3M").unwrap();
+    assert_eq!(2, duration.years());
+    assert_eq!(3, duration.months());
+    assert_eq!(27, duration.as_months());
+  }
 }
